import datetime
from django.core.management.base import BaseCommand
from django.db import transaction

from netbox_config_backup.tasks import backup_job


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--time', help="time")
        parser.add_argument('device', help="Device Name")

    def handle(self, *args, **options):
        from netbox_config_backup.models import Backup, BackupJob

        backupjob = BackupJob.objects.filter(backup__device__name=options['device']).last()
        backup_job(backupjob.pk)



# Generated by Django 4.0.3 on 2022-04-14 15:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_config_backup', '0010_backup_ip'),
    ]

    operations = [
        migrations.AlterField(
            model_name='backup',
            name='name',
            field=models.CharField(max_length=255, unique=True),
        ),
    ]



import datetime
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    def handle(self, *args, **options):
        from netbox_config_backup.git import repository
        from netbox_config_backup.models import Backup, BackupCommit, BackupFile, BackupObject, BackupCommitTreeChange
        LOCAL_TIMEZONE = datetime.datetime.now(datetime.timezone.utc).astimezone().tzinfo

        BackupCommitTreeChange.objects.all().delete()
        BackupCommit.objects.all().delete()
        BackupObject.objects.all().delete()

        print(f'Fetching Git log')
        log = reversed(repository.log())
        print(f'Fetched Git log')

        for entry in log:
            time = entry.get('time', datetime.datetime.now()).replace(tzinfo=LOCAL_TIMEZONE)

            try:
                bc = BackupCommit.objects.get(sha=entry.get('sha', None))
            except BackupCommit.DoesNotExist:
                bc = BackupCommit(sha=entry.get('sha', None), time=time)
            print(f'Saving commit: {bc.sha} at time {bc.time} with parent {entry.get("parents", [])}')
            bc.save()
            for change in entry.get('changes'):
                backup = None
                backupfile = None

                change_data = {}
                for key in ['old', 'new']:
                    sha = change.get(key, {}).get('sha', None)
                    file = change.get(key, {}).get('path', None)
                    if file is not None and file is not None:
                        uuid, type = file.split('.')
                        try:
                            backup = Backup.objects.get(uuid=uuid)
                        except Backup.DoesNotExist:
                            backup = Backup.objects.create(uuid=uuid, name=uuid)
                        try:
                            backupfile = BackupFile.objects.get(backup=backup, type=type)
                        except BackupFile.DoesNotExist:
                            backupfile = BackupFile.objects.create(backup=backup, type=type)
                        try:
                            object = BackupObject.objects.get(sha=sha)
                        except BackupObject.DoesNotExist:
                            object = BackupObject.objects.create(sha=sha)
                        change_data[key] = object

                    index = f'{key}-{sha}-{file}'
                    print(f'\t\t{index}')

                try:
                    bctc = BackupCommitTreeChange.objects.get(
                        backup=backup,
                        file=backupfile,
                        commit=bc,
                        type=change.get('type', None),
                        old=change_data.get('old', None),
                        new=change_data.get('new', None)
                    )
                except BackupCommitTreeChange.DoesNotExist:
                    bctc = BackupCommitTreeChange.objects.create(
                        backup=backup,
                        file=backupfile,
                        commit=bc,
                        type=change.get('type', None),
                        old=change_data.get('old', None),
                        new=change_data.get('new', None)
                    )

                newsha = bctc.new.sha if bctc.new else None
                oldsha = bctc.old.sha if bctc.old else None
                print(f'\tSaving change {bc.sha}, {oldsha}, {newsha}')
            print("")



import logging

from extras.plugins import PluginTemplateExtension

from netbox_config_backup.models import Backup, BackupJob, BackupCommitTreeChange
from netbox_config_backup.tables import BackupsTable
from netbox_config_backup.utils.backups import get_backup_tables
from utilities.htmx import is_htmx

logger = logging.getLogger(f"netbox_config_backup")


class DeviceBackups(PluginTemplateExtension):
    model = 'dcim.device'

    def full_width_page(self):
        request = self.context.get('request')
        def build_table(instance):
            bctc = BackupCommitTreeChange.objects.filter(
                backup=instance,
                file__isnull=False
            )
            table = BackupsTable(bctc, user=request.user)
            table.configure(request)

            return table

        device = self.context.get('object', None)
        devices = Backup.objects.filter(device=device) if device is not None else Backup.objects.none()
        if devices.count() > 0:
            instance = devices.first()
            table = build_table(instance)

            if BackupJob.is_queued(instance) is False:
                logger.debug(f'{instance}: Queuing Job')
                BackupJob.enqueue(instance)

            if is_htmx(request):
                return self.render('htmx/table.html', extra_context={
                    'object': instance,
                    'table': table,
                    'preferences': {},
                })
            return self.render('netbox_config_backup/inc/backup_tables.html', extra_context={
                'object': instance,
                'table': table,
                'preferences': request.user.config,
            })

        return ''


template_extensions = [DeviceBackups]



from django import forms
from django.forms import CharField
from django.utils.translation import gettext as _

from dcim.choices import DeviceStatusChoices
from dcim.models import Device
from ipam.models import IPAddress
from netbox_config_backup.models import Backup
from utilities.forms import BootstrapMixin
from utilities.forms.fields import DynamicModelChoiceField, DynamicModelMultipleChoiceField

__all__ = (
    'BackupForm',
    'BackupFilterSetForm',
)


class BackupForm(BootstrapMixin, forms.ModelForm):
    device = DynamicModelChoiceField(
        label='Device',
        required=False,
        queryset=Device.objects.all(),
        query_params={
            'status': [DeviceStatusChoices.STATUS_ACTIVE],
            'platform__napalm__ne': None,
            'has_primary_ip': True,
        },
    )
    ip = DynamicModelChoiceField(
        label='IP Address',
        required=False,
        queryset=IPAddress.objects.all(),
        query_params={
            'device_id': '$device'
        }
    )
    class Meta:
        model = Backup
        fields = ('name', 'device', 'status', 'ip')

    def clean(self):
        super().clean()
        if self.cleaned_data.get('device', None) == None:
            self.cleaned_data['ip'] = None


class BackupFilterSetForm(BootstrapMixin, forms.Form):
    model = Backup
    field_order = [
        'q', 'name', 'device_id', 'ip'
    ]
    q = forms.CharField(
        required=False,
        widget=forms.TextInput(attrs={'placeholder': _('All Fields')}),
        label=_('Search')
    )
    device_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        label=_('Device'),
        fetch_trigger='open',
        query_params={
            'status': [DeviceStatusChoices.STATUS_ACTIVE],
            'platform__napalm__ne': None,
            'has_primary_ip': True,
        }
    )
    ip = forms.CharField(
        required=False,
        widget=forms.TextInput(
            attrs={
                'placeholder': 'IP Address',
            }
        ),
        label=_('IP Address')
    )





from netbox_config_backup.models.backups import Backup, BackupCommit, BackupFile, BackupObject, BackupCommitTreeChange
from netbox_config_backup.models.jobs import BackupJob


__all__ = (
    'Backup',
    'BackupCommit',
    'BackupFile',
    'BackupObject',
    'BackupCommitTreeChange',
    'BackupJob',
)


from django.urls import path
from . import views

urlpatterns = [
    path('devices/', views.BackupListView.as_view(), name='backup_list'),
    path('devices/add/', views.BackupEditView.as_view(), name='backup_add'),
    path('devices/<int:pk>/', views.BackupView.as_view(), name='backup'),
    path('devices/<int:pk>/edit/', views.BackupEditView.as_view(), name='backup_edit'),
    path('devices/<int:pk>/delete/', views.BackupDeleteView.as_view(), name='backup_delete'),
    path('devices/<int:pk>/backups/', views.BackupBackupsView.as_view(), name='backup_backups'),
    path('devices/<int:pk>/config/', views.ConfigView.as_view(), name='backup_config'),
    path('devices/<int:pk>/config/<int:current>/', views.ConfigView.as_view(), name='backup_config'),
    path('devices/<int:pk>/diff/', views.DiffView.as_view(), name='backup_diff'),
    path('devices/<int:pk>/diff/<int:current>/', views.DiffView.as_view(), name='backup_diff'),
    path('devices/<int:pk>/diff/<int:current>/<int:previous>/', views.DiffView.as_view(), name='backup_diff'),
]



from extras.plugins import PluginMenuItem, PluginMenuButton, PluginMenu
from utilities.choices import ButtonColorChoices

item = PluginMenuItem(
        link='plugins:netbox_config_backup:backup_list',
        link_text='Devices',
        buttons=[
            PluginMenuButton(
                link="plugins:netbox_config_backup:backup_add",
                title="Add",
                icon_class="mdi mdi-plus",
                color=ButtonColorChoices.GREEN,
            ),
        ]
    )

menu = PluginMenu(
        label="Configuration Backup",
        groups=(
            ('Backup Jobs', (item,)),
        )
    )



from rest_framework import serializers

from dcim.api.nested_serializers import NestedDeviceSerializer
from ipam.api.nested_serializers import NestedIPAddressSerializer
from netbox.api.serializers import NetBoxModelSerializer

from netbox_config_backup.models import Backup


__all__ = (
    'BackupSerializer',
)


class BackupSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_sync-api:sync-detail')
    device = NestedDeviceSerializer()
    ip = NestedIPAddressSerializer()

    class Meta:
        model = Backup
        fields = [
            'id', 'url', 'display', 'device', 'ip', 'name', 'uuid', 'status'
        ]



import logging

from django.http import Http404
from django.shortcuts import get_object_or_404, render
from django.urls import reverse, NoReverseMatch
from django.views import View

from extras.choices import JobResultStatusChoices
from netbox.views.generic import ObjectDeleteView, ObjectEditView, ObjectView, ObjectListView, ObjectChildrenView
from netbox_config_backup.filtersets import BackupFilterSet, BackupsFilterSet

from netbox_config_backup.forms import BackupForm, BackupFilterSetForm
from netbox_config_backup.git import GitBackup
from netbox_config_backup.models import Backup, BackupJob, BackupCommitTreeChange, BackupCommit, BackupObject
from netbox_config_backup.tables import BackupTable, BackupsTable
from netbox_config_backup.utils import get_backup_tables, Differ
from utilities.views import register_model_view, ViewTab

logger = logging.getLogger(f"netbox_config_backup")


class BackupListView(ObjectListView):
    queryset = Backup.objects.all()
    filterset = BackupFilterSet
    filterset_form = BackupFilterSetForm
    table = BackupTable
    action_buttons = ('add',)


@register_model_view(Backup)
class BackupView(ObjectView):
    queryset = Backup.objects.all()
    template_name = 'netbox_config_backup/backup.html'

    def get_extra_context(self, request, instance):

        jobs = BackupJob.objects.filter(backup=instance).order_by()
        is_running = True if jobs.filter(status=JobResultStatusChoices.STATUS_RUNNING).count() > 0 else False
        is_pending = True if jobs.filter(status=JobResultStatusChoices.STATUS_PENDING).count() > 0 else False

        job_status = None
        if is_pending:
            job_status = 'Pending'
        if is_running:
            job_status = 'Running'

        if BackupJob.is_queued(instance) is False:
            logger.debug(f'{instance}: Queuing Job')
            BackupJob.enqueue_if_needed(instance)

        status = {
            'status': job_status,
            'scheduled': BackupJob.is_queued(instance),
            'next_attempt': instance.next_attempt,
            'last_job': instance.jobs.filter(completed__isnull=False).last(),
            'last_success': instance.last_backup,
            'last_change': instance.last_change,
        }

        return {
            'status': status,
        }


@register_model_view(Backup, name='backups')
class BackupBackupsView(ObjectChildrenView):
    queryset = Backup.objects.all()
    template_name = 'netbox_config_backup/backups.html'
    child_model = BackupCommitTreeChange
    table = BackupsTable
    filterset = BackupsFilterSet
    actions = ['config', 'diff']
    action_perms = {
        'config': {'view'},
        'diff': {'view'},
    }
    tab = ViewTab(
        label='View Backups',
        badge=lambda obj: BackupCommitTreeChange.objects.filter(backup=obj, file__isnull=False).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(backup=parent, file__isnull=False)

    def get_extra_context(self, request, instance):
        return {
            'running': bool(request.GET.get('type') == 'running'),
            'startup': bool(request.GET.get('type') == 'startup'),
        }




@register_model_view(Backup, 'edit')
class BackupEditView(ObjectEditView):
    queryset = Backup.objects.all()
    form = BackupForm


@register_model_view(Backup, 'delete')
class BackupDeleteView(ObjectDeleteView):
    queryset = Backup.objects.all()

    def get_return_url(self, request, obj=None):

        # First, see if `return_url` was specified as a query parameter or form data. Use this URL only if it's
        # considered safe.
        return_url = request.GET.get('return_url') or request.POST.get('return_url')
        if return_url and return_url.startswith('/'):
            return return_url

        # Next, check if the object being modified (if any) has an absolute URL.
        if obj is not None and obj.pk and hasattr(obj, 'get_absolute_url'):
            return obj.get_absolute_url()

        # Fall back to the default URL (if specified) for the view.
        if self.default_return_url is not None:
            return reverse(self.default_return_url)

        # Attempt to dynamically resolve the list view for the object
        if hasattr(self, 'queryset'):
            model_opts = self.queryset.model._meta
            try:
                return reverse(f'plugins:{model_opts.app_label}:{model_opts.model_name}_list')
            except NoReverseMatch:
                pass

        # If all else fails, return home. Ideally this should never happen.
        return reverse('home')


@register_model_view(Backup, 'config')
class ConfigView(ObjectView):
    queryset = Backup.objects.all()
    template_name = 'netbox_config_backup/config.html'
    tab = ViewTab(
        label='Configuration',
    )

    def get(self, request, pk, current=None):
        backup = get_object_or_404(Backup.objects.all(), pk=pk)
        if current:
            current = get_object_or_404(BackupCommitTreeChange.objects.all(), pk=current)
        else:
            current = BackupCommitTreeChange.objects.filter(backup=backup, file__isnull=False).last()
            if not current:
                raise Http404(
                    "No current commit available"
                )

        path = f'{current.file.path}'

        repo = GitBackup()
        config = repo.read(path, current.commit.sha)

        previous = None
        if current is not None and current.old is not None:
            try:
                previous = backup.changes.filter(file__type=current.file.type, commit__time__lt=current.commit.time).\
                    last()
            except:
                pass

        return render(request, 'netbox_config_backup/config.html', {
            'object': backup,
            'tab': self.tab,
            'backup_config': config,
            'current': current,
            'previous': previous,
            'active_tab': 'config',
        })


@register_model_view(Backup, 'diff')
class DiffView(ObjectView):
    queryset = Backup.objects.all()
    template_name = 'netbox_config_backup/diff.html'
    tab = ViewTab(
        label='Diff',
    )

    def get(self, request, pk, current=None, previous=None):
        backup = get_object_or_404(Backup.objects.all(), pk=pk)
        if current:
            current = get_object_or_404(BackupCommitTreeChange.objects.all(), pk=current)
        else:
            current = BackupCommitTreeChange.objects.filter(backup=backup, file__isnull=False).last()
            if not current:
                raise Http404(
                    "No current commit available"
                )
        if previous:
            previous = get_object_or_404(BackupCommitTreeChange.objects.all(), pk=previous)
        else:
            previous = BackupCommitTreeChange.objects.filter(
                backup=backup,
                file__type=current.file.type,
                commit__time__lt=current.commit.time
            ).last()
            if not previous:
                raise Http404(
                    "No Previous Commit"
                )

        path = f'{current.file.path}'

        repo = GitBackup()

        previous_sha = previous.commit.sha if previous.commit is not None else 'HEAD'
        current_sha = current.commit.sha if current.commit is not None else None

        if backup.device and backup.device.platform.napalm_driver in ['ios', 'nxos']:
            new = repo.read(path, current_sha)
            old = repo.read(path, previous_sha)
            differ = Differ(old, new)
            diff = differ.cisco_compare()
        else:
            new = repo.read(path, current_sha)
            old = repo.read(path, previous_sha)
            differ = Differ(old, new)
            diff = differ.compare()

        for idx, line in enumerate(diff):
            diff[idx] = line.rstrip()

        return render(request, 'netbox_config_backup/diff.html', {
            'object': backup,
            'tab': self.tab,
            'diff': diff,
            'current': current,
            'previous': previous,
            'active_tab': 'diff',
        })



# Generated by Django 3.2.8 on 2021-11-23 17:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_config_backup', '0005_commit_add_time_field'),
    ]

    operations = [
        migrations.AlterField(
            model_name='backupcommit',
            name='backup',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='commits', to='netbox_config_backup.backup'),
        ),
    ]



# Generated by Django 3.2.8 on 2021-11-23 18:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_config_backup', '0007_backup_job_add_scheduled'),
    ]

    operations = [
        migrations.AlterField(
            model_name='backupjob',
            name='scheduled',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



# Generated by Django 4.1.7 on 2023-04-05 19:31

from django.db import migrations, models
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    dependencies = [
        ('extras', '0084_staging'),
        ('netbox_config_backup', '0012_backup_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='backup',
            name='created',
            field=models.DateTimeField(auto_now_add=True, null=True),
        ),
        migrations.AddField(
            model_name='backup',
            name='custom_field_data',
            field=models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder),
        ),
        migrations.AddField(
            model_name='backup',
            name='last_updated',
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AddField(
            model_name='backup',
            name='tags',
            field=taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag'),
        ),
        migrations.AlterField(
            model_name='backup',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False),
        ),
    ]



# Generated by Django 3.2.8 on 2021-11-20 03:24

import django.core.serializers.json
from django.db import migrations, models
import django.db.models.deletion
import taggit.managers
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('dcim', '0133_port_colors'),
        ('extras', '0062_clear_secrets_changelog'),
    ]

    operations = [
        migrations.CreateModel(
            name='Backup',
            fields=[
                ('created', models.DateField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('device', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='dcim.device')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='BackupJob',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('started', models.DateTimeField(blank=True, null=True)),
                ('completed', models.DateTimeField(blank=True, null=True)),
                ('status', models.CharField(default='pending', max_length=30)),
                ('data', models.JSONField(blank=True, null=True)),
                ('job_id', models.UUIDField(unique=True)),
                ('backup', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='jobs', to='netbox_config_backup.backup')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.2.8 on 2021-11-22 23:14

import django.core.serializers.json
from django.db import migrations, models
import django.db.models.deletion
import taggit.managers


class Migration(migrations.Migration):

    dependencies = [
        ('extras', '0062_clear_secrets_changelog'),
        ('netbox_config_backup', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BackupCommit',
            fields=[
                ('created', models.DateField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('sha', models.CharField(max_length=64)),
                ('backup', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='netbox_config_backup.backup')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterField(
            model_name='backupjob',
            name='backup',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='jobs', to='netbox_config_backup.backup'),
        ),
        migrations.CreateModel(
            name='BackupObject',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('sha', models.CharField(max_length=64)),
                ('file', models.CharField(max_length=255)),
            ],
            options={
                'unique_together': {('sha', 'file')},
            },
        ),
        migrations.CreateModel(
            name='BackupCommitTreeChange',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('type', models.CharField(max_length=10)),
                ('commit', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='changes', to='netbox_config_backup.backupcommit')),
                ('new', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='new', to='netbox_config_backup.backupobject')),
                ('old', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='previous', to='netbox_config_backup.backupobject')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from setuptools import find_packages, setup

setup(
    name='netbox_config_backup',
    version='1.4.5',
    description='NetBox Configuration Backup',
    long_description='Plugin to backup device configuration',
    url='https://github.com/dansheps/netbox-config-backup/',
    download_url='https://github.com/dansheps/netbox-config-backup/',
    author='Daniel Sheppard',
    author_email='dans@dansheps.com',
    maintainer='Daniel Sheppard',
    maintainer_email='dans@dansheps.com',
    install_requires=[
        'napalm',
        'uuid',
        'dulwich',
        'pydriller',
        'deepdiff',
    ],
    packages=find_packages(),
    include_package_data=True,
    license='Proprietary',
    zip_safe=False,
    platform=[],
    keywords=['netbox', 'netbox-plugin'],
    classifiers=[
        'Framework :: Django',
        'Programming Language :: Python :: 3',
    ]
)


# Generated by Django 3.2.8 on 2021-11-23 04:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_config_backup', '0004_custom_constraints'),
    ]

    operations = [
        migrations.AddField(
            model_name='backupcommit',
            name='time',
            field=models.DateTimeField(auto_now=True),
        ),
    ]



# Generated by Django 3.2.8 on 2021-11-23 17:54

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_config_backup', '0006_backup_add_commit_last_time'),
    ]

    operations = [
        migrations.AddField(
            model_name='backupjob',
            name='scheduled',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.8 on 2021-11-23 03:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_config_backup', '0003_primary_model_to_bigid'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='backupcommit',
            constraint=models.CheckConstraint(check=models.Q(('backup__isnull', False), ('sha__isnull', False)), name='backup_and_sha_not_null'),
        ),
    ]
